# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-22 21:16
from __future__ import unicode_literals

import calaccess_raw
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0008_auto_20160707_0518'),
    ]

    operations = [
        migrations.AddField(
            model_name='rawdataversion',
            name='clean_zip_archive',
            field=models.FileField(blank=True, help_text='An archive zip of cleaned (and error log) files', max_length=255, upload_to=calaccess_raw.archive_directory_path, verbose_name='cleaned files zip archive'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-03 06:00
from __future__ import unicode_literals

import calaccess_raw.annotations
import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='acronymscd',
            options={'ordering': ('acronym',)},
        ),
        migrations.AlterModelOptions(
            name='addresscd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='ballotmeasurescd',
            options={'ordering': ('-election_date', 'measure_no', 'measure_short_name', 'measure_name')},
        ),
        migrations.AlterModelOptions(
            name='cvr2campaigndisclosurecd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='cvr2lobbydisclosurecd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='cvr2registrationcd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='cvr2socd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='cvr3verificationinfocd',
            options={'ordering': ('-sig_date',)},
        ),
        migrations.AlterModelOptions(
            name='cvrcampaigndisclosurecd',
            options={'ordering': ('-rpt_date',)},
        ),
        migrations.AlterModelOptions(
            name='cvre530cd',
            options={'ordering': ('-pmnt_dt',)},
        ),
        migrations.AlterModelOptions(
            name='cvrf470cd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='cvrlobbydisclosurecd',
            options={'ordering': ('-rpt_date',)},
        ),
        migrations.AlterModelOptions(
            name='cvrregistrationcd',
            options={'ordering': ('-rpt_date',)},
        ),
        migrations.AlterModelOptions(
            name='cvrsocd',
            options={'ordering': ('-rpt_date',)},
        ),
        migrations.AlterModelOptions(
            name='debtcd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='efsfilinglogcd',
            options={'ordering': ('-filing_date',)},
        ),
        migrations.AlterModelOptions(
            name='expncd',
            options={'ordering': ('-expn_date',)},
        ),
        migrations.AlterModelOptions(
            name='f495p2cd',
            options={'ordering': ('-elect_date',)},
        ),
        migrations.AlterModelOptions(
            name='f501502cd',
            options={'ordering': ('-rpt_date',)},
        ),
        migrations.AlterModelOptions(
            name='f690p2cd',
            options={'ordering': ('-exec_date',)},
        ),
        migrations.AlterModelOptions(
            name='fileracronymscd',
            options={'ordering': ('acronym',)},
        ),
        migrations.AlterModelOptions(
            name='fileraddresscd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='filerethicsclasscd',
            options={'ordering': ('-ethics_date',)},
        ),
        migrations.AlterModelOptions(
            name='filerfilingscd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='filerinterestscd',
            options={'ordering': ('-effect_date',)},
        ),
        migrations.AlterModelOptions(
            name='filerlinkscd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='filernamecd',
            options={'ordering': ('naml', 'namf')},
        ),
        migrations.AlterModelOptions(
            name='filerscd',
            options={'ordering': ('-filer_id',)},
        ),
        migrations.AlterModelOptions(
            name='filerstatustypescd',
            options={'ordering': ('status_type',)},
        ),
        migrations.AlterModelOptions(
            name='filertofilertypecd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='filertypeperiodscd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='filertypescd',
            options={'ordering': ('filer_type',)},
        ),
        migrations.AlterModelOptions(
            name='filerxrefcd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='filingperiodcd',
            options={'ordering': ('-end_date',)},
        ),
        migrations.AlterModelOptions(
            name='filingscd',
            options={'ordering': ('-filing_id',)},
        ),
        migrations.AlterModelOptions(
            name='grouptypescd',
            options={'ordering': ('grp_id',)},
        ),
        migrations.AlterModelOptions(
            name='hdrcd',
            options={'ordering': ('-filing_id', '-amend_id')},
        ),
        migrations.AlterModelOptions(
            name='headercd',
            options={'ordering': ('form_id', 'line_number')},
        ),
        migrations.AlterModelOptions(
            name='imagelinkscd',
            options={'ordering': ('-img_dt',)},
        ),
        migrations.AlterModelOptions(
            name='lattcd',
            options={'ordering': ('-pmt_date',)},
        ),
        migrations.AlterModelOptions(
            name='lccmcd',
            options={'ordering': ('-ctrib_date',)},
        ),
        migrations.AlterModelOptions(
            name='legislativesessionscd',
            options={'ordering': ('-begin_date',)},
        ),
        migrations.AlterModelOptions(
            name='lempcd',
            options={'ordering': ('-eff_date',)},
        ),
        migrations.AlterModelOptions(
            name='lexpcd',
            options={'ordering': ('-expn_date',)},
        ),
        migrations.AlterModelOptions(
            name='loancd',
            options={'ordering': ('-loan_date1',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyamendmentscd',
            options={'ordering': ('-exec_date',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyingchglogcd',
            options={'ordering': ('-log_dt',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistcontributions1cd',
            options={'ordering': ('-filing_period_start_dt',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistcontributions2cd',
            options={'ordering': ('-filing_period_start_dt',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistcontributions3cd',
            options={'ordering': ('-filing_period_start_dt',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistemplobbyist1cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistemplobbyist2cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistemployer1cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistemployer2cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistemployer3cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistemployerfirms1cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistemployerfirms2cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistemployerhistorycd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='lobbyistfirm1cd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='lobbyistfirm2cd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='lobbyistfirm3cd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='lobbyistfirmemployer1cd',
            options={'ordering': ('-rpt_start',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistfirmemployer2cd',
            options={'ordering': ('-rpt_start',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistfirmhistorycd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='lobbyistfirmlobbyist1cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lobbyistfirmlobbyist2cd',
            options={'ordering': ('-session_id',)},
        ),
        migrations.AlterModelOptions(
            name='lookupcodescd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='lothcd',
            options={'ordering': ('-pmt_date',)},
        ),
        migrations.AlterModelOptions(
            name='lpaycd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='namescd',
            options={'ordering': ('naml', 'namf')},
        ),
        migrations.AlterModelOptions(
            name='rcptcd',
            options={'ordering': ('-rcpt_date',)},
        ),
        migrations.AlterModelOptions(
            name='receivedfilingscd',
            options={'ordering': ('-received_date',)},
        ),
        migrations.AlterModelOptions(
            name='reportscd',
            options={'ordering': ('rpt_name',)},
        ),
        migrations.AlterModelOptions(
            name='s401cd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='s496cd',
            options={'ordering': ('-exp_date',)},
        ),
        migrations.AlterModelOptions(
            name='s497cd',
            options={'ordering': ('-ctrib_date',)},
        ),
        migrations.AlterModelOptions(
            name='s498cd',
            options={'ordering': ('-date_rcvd',)},
        ),
        migrations.AlterModelOptions(
            name='smrycd',
            options={'ordering': ('filing_id', '-amend_id', 'form_type', 'line_item')},
        ),
        migrations.AlterModelOptions(
            name='spltcd',
            options={},
        ),
        migrations.AlterModelOptions(
            name='textmemocd',
            options={},
        ),
        migrations.AlterField(
            model_name='cvre530cd',
            name='filing_id',
            field=calaccess_raw.fields.IntegerField(db_column='FILING_ID', db_index=True, help_text='Unique filing identification number', verbose_name='filing ID'),
        ),
        migrations.AlterField(
            model_name='filerfilingscd',
            name='filing_id',
            field=calaccess_raw.fields.IntegerField(db_column='FILING_ID', db_index=True, help_text='Unique filing identification number', verbose_name='filing ID'),
        ),
        migrations.AlterField(
            model_name='filernamecd',
            name='effect_dt',
            field=calaccess_raw.fields.DateField(db_column='EFFECT_DT', help_text='effective date for status', null=True),
        ),
        migrations.AlterField(
            model_name='filernamecd',
            name='filer_id',
            field=calaccess_raw.fields.IntegerField(db_column='FILER_ID', db_index=True, help_text="filer's unique identification number", null=True, verbose_name='filer ID'),
        ),
        migrations.AlterField(
            model_name='filernamecd',
            name='xref_filer_id',
            field=calaccess_raw.fields.CharField(db_column='XREF_FILER_ID', db_index=True, help_text='alternative filer ID found on many forms', max_length=15, verbose_name='crossreference filer ID'),
        ),
        migrations.AlterField(
            model_name='filingscd',
            name='filing_id',
            field=calaccess_raw.fields.IntegerField(db_column='FILING_ID', db_index=True, help_text='Unique filing identification number', verbose_name='filing ID'),
        ),
        migrations.AlterField(
            model_name='hdrcd',
            name='filing_id',
            field=calaccess_raw.fields.IntegerField(db_column='FILING_ID', db_index=True, help_text='Unique filing identification number', verbose_name='filing ID'),
        ),
        migrations.AlterField(
            model_name='receivedfilingscd',
            name='filing_id',
            field=calaccess_raw.fields.IntegerField(blank=True, db_column='FILING_ID', db_index=True, help_text='Unique filing identification number', null=True, verbose_name='filing ID'),
        ),
        migrations.AlterField(
            model_name='smrycd',
            name='filing_id',
            field=calaccess_raw.fields.IntegerField(db_column='FILING_ID', db_index=True, help_text='Unique filing identification number', verbose_name='filing ID'),
        ),
        migrations.AlterField(
            model_name='spltcd',
            name='amend_id',
            field=calaccess_raw.fields.IntegerField(db_column='AMEND_ID', db_index=True, help_text='amendment identification number. A number of 0 is the original filing and 1 to 999 amendments.', verbose_name='amendment ID'),
        ),
        migrations.AlterField(
            model_name='spltcd',
            name='elec_amount',
            field=calaccess_raw.fields.DecimalField(db_column='ELEC_AMOUNT', decimal_places=2, help_text='per election to date amount', max_digits=16, verbose_name='election amount'),
        ),
        migrations.AlterField(
            model_name='spltcd',
            name='elec_code',
            field=calaccess_raw.fields.CharField(blank=True, choices=[('P', 'Primary'), ('G', 'General'), ('S', 'Special'), ('R', 'Runoff'), ('g', 'General'), ('p', 'primary'), ('C', 'Unknown'), ('D', 'Unknown'), ('F', 'Unknown'), ('M', 'Unknown'), ('N', 'Unknown'), ('X', 'Unknown'), ('O', 'Unknown'), ('0', 'Unknown'), ('1', 'Unknown'), ('2', 'Unknown')], db_column='ELEC_CODE', documentcloud_pages=[calaccess_raw.annotations.DocumentCloud(id='2712034-Cal-Format-201', start_page=18)], help_text='per election to date code', max_length=2, verbose_name='election code'),
        ),
        migrations.AlterField(
            model_name='spltcd',
            name='elec_date',
            field=calaccess_raw.fields.DateField(db_column='ELEC_DATE', help_text='date of election', null=True),
        ),
        migrations.AlterField(
            model_name='spltcd',
            name='filing_id',
            field=calaccess_raw.fields.IntegerField(db_column='FILING_ID', db_index=True, help_text='unique filing identification number', verbose_name='filing ID'),
        ),
        migrations.AlterField(
            model_name='spltcd',
            name='line_item',
            field=calaccess_raw.fields.IntegerField(db_column='LINE_ITEM', db_index=True, help_text='line item number of this record'),
        ),
        migrations.AlterField(
            model_name='spltcd',
            name='ptran_id',
            field=calaccess_raw.fields.CharField(blank=True, db_column='PTRAN_ID', help_text='parent transaction ID', max_length=32, verbose_name='parent transaction ID'),
        ),
        migrations.AlterField(
            model_name='textmemocd',
            name='filing_id',
            field=calaccess_raw.fields.IntegerField(db_column='FILING_ID', db_index=True, help_text='Unique filing identification number', verbose_name='filing ID'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Load clean CAL-ACCESS CSV file into a database model.
"""
from __future__ import unicode_literals
import os
import six
from django.apps import apps
from csvkit import CSVKitReader
from django.conf import settings
from postgres_copy import CopyMapping
from django.db import connections, router
from django.core.management.base import CommandError
from django.utils.timezone import now
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw.models.tracking import RawDataFile


class Command(CalAccessCommand):
    """
    Load clean CAL-ACCESS CSV file into a database model.
    """
    help = 'Load clean CAL-ACCESS CSV file into a database model'
    # Trick for reformating date strings in source data so that they can
    # be gobbled up by MySQL. You'll see how below.
    date_sql = "DATE_FORMAT(str_to_date(@`%s`, '%%c/%%e/%%Y'), '%%Y-%%m-%%d')"
    datetime_sql = "DATE_FORMAT(str_to_date(@`%s`, '%%c/%%e/%%Y \
%%h:%%i:%%s %%p'), '%%Y-%%m-%%d  %%H:%%i:%%s')"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        # positional (required) arg
        parser.add_argument(
            'model_name',
            help="Name of the model into which data will be loaded"
        )
        # keyword (optional) args
        parser.add_argument(
            "--c",
            "--csv",
            dest='csv',
            default=None,
            help="Path to comma-delimited file to be loaded. Defaults to one associated with model."
        )
        parser.add_argument(
            "--keep-file",
            action="store_true",
            dest="keep_file",
            default=False,
            help="Keep clean CSV file after loading"
        )
        parser.add_argument(
            "-a",
            "--app-name",
            dest="app_name",
            default="calaccess_raw",
            help="Name of Django app with models into which data will "
                 "be imported (if other not calaccess_raw)"
        )

    # all BaseCommand subclasses require a handle() method that includes
    #   the actual logic of the command
    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # set / compute any attributes that multiple class methods need
        self.keep_file = options["keep_file"]
        # get model based on strings of app_name and model_name
        self.model = apps.get_model(options["app_name"], options['model_name'])

        # load from provided csv or csv mapped to model
        self.csv = options["csv"] or self.model.objects.get_csv_path()

        # load into database suggested for model by router
        self.database = router.db_for_write(model=self.model)

        # get most recently cleaned RawDataFile
        try:
            raw_file = RawDataFile.objects.filter(
                file_name=self.model._meta.db_table,
                clean_start_datetime__isnull=False
            ).latest('clean_start_datetime')
        except RawDataFile.DoesNotExist:
            raise CommandError(
                'No record of cleaning {0}.TSV (run `python manage.py '
                'cleancalaccessrawfile {0}`).'.format(self.model._meta.db_table)
            )
        # raise exception if clean step did not finish
        if not raw_file.clean_finish_datetime:
            raise CommandError(
                'Previous cleaning of {0}.TSV did not finish (run `python manage.py '
                'cleancalaccessrawfile {0}`).'.format(self.model._meta.db_table)
            )

        # get the row count
        row_count = self.get_row_count()

        if row_count == 0:
            if self.verbosity > 2:
                self.failure("%s is empty." % self.csv)
        else:
            # store the start time for the load
            raw_file.load_start_datetime = now()
            # reset the finish time for the load
            raw_file.load_finish_datetime = None
            # save here in case command doesn't finish
            raw_file.save()

            if self.verbosity > 2:
                self.log(" Loading %s" % options['model_name'])
            self.load()

            # add load counts to raw_file_record
            raw_file.load_columns_count = len(self.model._meta.fields)
            raw_file.load_records_count = self.model.objects.count()

            # if not keeping files, remove the csv file
            if not self.keep_file:
                os.remove(self.csv)

            # store the finish time for the load
            raw_file.load_finish_datetime = now()
            # and save the RawDataFile
            raw_file.save()

    def load(self):
        """
        Loads the source CSV for the provided model based on settings and database connections.
        """
        # check if can load into dat
        if getattr(settings, 'CALACCESS_DAT_SOURCE', None) and six.PY2:
            self.load_dat()

        # if not using default db, make sure the database is set up in django's settings
        if self.database:
            try:
                engine = settings.DATABASES[self.database]['ENGINE']
            except KeyError:
                raise TypeError(
                    "{} not configured in DATABASES settings.".format(self.database)
                )

        # set up database connection
        self.connection = connections[self.database]
        self.cursor = self.connection.cursor()

        # check the kind of database before calling db-specific load method
        if engine == 'django.db.backends.mysql':
            self.load_mysql()
        elif engine in (
            'django.db.backends.postgresql_psycopg2'
            'django.contrib.gis.db.backends.postgis'
        ):
            self.load_postgresql()
        else:
            self.failure("Sorry your database engine is unsupported")
            raise CommandError(
                "Only MySQL and PostgresSQL backends supported."
            )

    def load_dat(self):
        """
        Takes a model and a csv file and loads it into dat.
        """
        import datpy
        dat_source = settings.CALACCESS_DAT_SOURCE
        self.dat = datpy.Dat(dat_source['source'])
        dataset = self.dat.dataset(self.model._meta.db_table)
        try:
            dataset.import_file(self.csv, format='csv')
            if self.verbosity > 2:
                dat_status = self.dat.status()
                model_count = dat_status['rows']
                csv_count = self.get_row_count(self.csv)
                self.finish_load_message(model_count, csv_count)
        except datpy.DatException:
            raise CommandError(
                'Failed to load dat for %s, %s' % (
                    self.model._meta.db_table,
                    self.csv
                )
            )

    def load_mysql(self):
        """
        Load the file into a MySQL database using LOAD DATA INFILE.
        """
        import warnings
        import MySQLdb
        warnings.filterwarnings("ignore", category=MySQLdb.Warning)

        # Flush the target model
        self.cursor.execute('TRUNCATE TABLE %s' % self.model._meta.db_table)

        # Build the MySQL LOAD DATA INFILE command
        bulk_sql_load_part_1 = """
            LOAD DATA LOCAL INFILE '%s'
            INTO TABLE %s
            FIELDS TERMINATED BY ','
            OPTIONALLY ENCLOSED BY '"'
            LINES TERMINATED BY '\\n'
            IGNORE 1 LINES
            (
        """ % (
            self.csv,
            self.model._meta.db_table
        )

        # Get the headers and the row count from the source CSV
        csv_headers = self.get_headers()
        csv_record_cnt = self.get_row_count()

        header_sql_list = []
        field_types = dict(
            (f.db_column, f.db_type(self.connection))
            for f in self.model._meta.fields
        )
        date_set_list = []
        char_set_list = []

        for h in csv_headers:
            # Pull the data type of the field
            data_type = field_types[h]
            # If it is a date field, we need to reformat the data
            # so that MySQL will properly parse it on the way in.
            if data_type == 'date':
                header_sql_list.append('@`%s`' % h)
                date_set_list.append(
                    "`%s` =  %s" % (h, self.date_sql % h)
                )
            elif data_type == 'datetime':
                header_sql_list.append('@`%s`' % h)
                date_set_list.append(
                    "`%s` =  %s" % (h, self.datetime_sql % h)
                )
            elif 'char' in data_type:
                header_sql_list.append('@`%s`' % h)
                char_set_list.append(
                    r"`{0}` = TRIM(TRAILING '\n' FROM @`{0}`)".format(h)
                )
            else:
                header_sql_list.append('`%s`' % h)

        bulk_sql_load = bulk_sql_load_part_1 + ','.join(header_sql_list) + ')'
        if date_set_list or char_set_list:
            bulk_sql_load += " set %s" % ",".join(date_set_list + char_set_list)

        # Run the query
        cnt = self.cursor.execute(bulk_sql_load)

        # Report back on how we did
        if self.verbosity > 2:
            self.finish_load_message(cnt, csv_record_cnt)

    def load_postgresql(self):
        """
        Load the file into a PostgreSQL database using COPY.
        """
        # Drop all the records from the target model's real table
        self.cursor.execute('TRUNCATE TABLE "%s" CASCADE' % (
            self.model._meta.db_table
        ))

        c = CopyMapping(
            self.model,
            self.csv,
            dict((f.name, f.db_column) for f in self.model._meta.fields),
            using=self.database,
        )
        c.save(silent=True)

        # Print out the results
        if self.verbosity > 2:
            csv_count = self.get_row_count()
            model_count = self.model.objects.count()
            self.finish_load_message(model_count, csv_count)

    def get_headers(self):
        """
        Returns the column headers from the csv as a list.
        """
        with open(self.csv, 'r') as infile:
            csv_reader = CSVKitReader(infile)
            try:
                headers = next(csv_reader)
            except StopIteration:
                headers = []
        return headers

    def get_row_count(self):
        """
        Returns the number of rows in the file, not counting headers.
        """
        with open(self.csv) as infile:
            row_count = sum(1 for line in infile) - 1

        if row_count < 0:
            row_count = 0

        return row_count

    def finish_load_message(self, model_count, csv_count):
        """
        The message displayed about whether or not a load finished successfully.
        """
        if model_count != csv_count:
            msg = '  Table record count doesn\'t match CSV. \
Table: %s\tCSV: %s'
            self.failure(msg % (
                model_count,
                csv_count,
            ))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-01 20:39
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0013_auto_20160729_0506'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='rawdatacommand',
            name='called_by',
        ),
        migrations.RemoveField(
            model_name='rawdatacommand',
            name='version',
        ),
        migrations.DeleteModel(
            name='RawDataCommand',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-28 19:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0010_auto_20160726_1425'),
    ]

    operations = [
        migrations.AddField(
            model_name='rawdatafile',
            name='clean_finish_datetime',
            field=models.DateTimeField(help_text='Date and time when the cleaning of the file finished', null=True, verbose_name='date and time cleaning finished'),
        ),
        migrations.AddField(
            model_name='rawdatafile',
            name='clean_start_datetime',
            field=models.DateTimeField(help_text='Date and time when the cleaning of the file started', null=True, verbose_name='date and time cleaning started'),
        ),
        migrations.AddField(
            model_name='rawdatafile',
            name='load_finish_datetime',
            field=models.DateTimeField(help_text='Date and time when the loading of the file finished', null=True, verbose_name='date and time extraction finished'),
        ),
        migrations.AddField(
            model_name='rawdatafile',
            name='load_start_datetime',
            field=models.DateTimeField(help_text='Date and time when the loading of the file started', null=True, verbose_name='date and time loading started'),
        ),
        migrations.AddField(
            model_name='rawdataversion',
            name='download_finish_datetime',
            field=models.DateTimeField(help_text='Date and time when the download of the CAL-ACCESS database export finished', null=True, verbose_name='date and time download finished'),
        ),
        migrations.AddField(
            model_name='rawdataversion',
            name='download_start_datetime',
            field=models.DateTimeField(help_text='Date and time when the download of the CAL-ACCESS database export started', null=True, verbose_name='date and time download started'),
        ),
        migrations.AddField(
            model_name='rawdataversion',
            name='extract_finish_datetime',
            field=models.DateTimeField(help_text='Date and time when extraction of the CAL-ACCESS data files finished', null=True, verbose_name='date and time extraction finished'),
        ),
        migrations.AddField(
            model_name='rawdataversion',
            name='extract_start_datetime',
            field=models.DateTimeField(help_text='Date and time when extraction of the CAL-ACCESS data files started', null=True, verbose_name='date and time extraction started'),
        ),
        migrations.AddField(
            model_name='rawdataversion',
            name='update_finish_datetime',
            field=models.DateTimeField(help_text='Date and time when the update to the CAL-ACCESS version finished', null=True, verbose_name='date and time update finished'),
        ),
        migrations.AddField(
            model_name='rawdataversion',
            name='update_start_datetime',
            field=models.DateTimeField(help_text='Date and time when the update to the CAL-ACCESS version started', null=True, verbose_name='date and time update started'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for tracking models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_raw import models
from .base import BaseAdmin


@admin.register(models.RawDataVersion)
class RawDataVersionAdmin(BaseAdmin):
    """
    Custom admin for the RawDataVersion model.
    """
    list_display = (
        "id",
        "release_datetime",
        "pretty_download_size",
        "download_file_count",
        "download_record_count",
        "clean_file_count",
        "clean_record_count",
        "pretty_clean_size",
    )
    list_display_links = ('release_datetime',)
    list_filter = ("release_datetime",)


@admin.register(models.RawDataFile)
class RawDataFileAdmin(BaseAdmin):
    """
    Custom admin for the RawDataFile model.
    """
    list_display = (
        "id",
        "version",
        "file_name",
        "download_records_count",
        "clean_records_count",
        "load_records_count",
        "error_count"
    )
    list_display_links = ('id', 'file_name',)
    list_filter = ("version__release_datetime",)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Download the latest CAL-ACCESS database ZIP.
"""
from __future__ import unicode_literals
import os
import logging
import requests
from datetime import datetime
from hurry.filesize import size
from clint.textui import progress
from django.conf import settings
from django.contrib.humanize.templatetags.humanize import naturaltime
from django.core.files import File
from django.utils.timezone import utc, now
from calaccess_raw import get_download_directory
from django.template.loader import render_to_string
from django.core.management.base import CommandError
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw.models.tracking import RawDataVersion
logger = logging.getLogger(__name__)


class Command(CalAccessCommand):
    """
    Download the latest CAL-ACCESS database ZIP.
    """
    help = "Download the latest CAL-ACCESS database ZIP"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--noinput",
            action="store_true",
            dest="noinput",
            default=False,
            help="Download the ZIP archive without asking permission"
        )
        parser.add_argument(
            "--force-restart",
            "--restart",
            action="store_true",
            dest="restart",
            default=False,
            help="Force re-start (overrides auto-resume)."
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # get the dir where data goes from app settings
        self.data_dir = get_download_directory()
        # if data_dir doesn't exist, create it
        os.path.exists(self.data_dir) or os.makedirs(self.data_dir)

        # downloaded zip file will go in data_dir
        self.zip_path = os.path.join(self.data_dir, self.url.split('/')[-1])

        self.download_metadata = self.get_download_metadata()
        logger.debug('Server: %s <-- (from HEAD)' % self.download_metadata['server'])
        logger.debug('ETag: %s <-- (from HEAD)' % self.download_metadata['etag'])
        logger.debug(
            'Last-Modified: %s <-- (from HEAD)' % self.download_metadata['last-modified']
        )
        logger.debug(
            'Content-Length %s <-- (from HEAD)' % self.download_metadata['content-length']
        )

        # get or create the RawDataVersion
        self.version, created = RawDataVersion.objects.get_or_create(
            release_datetime=self.download_metadata['last-modified'],
            expected_size=self.download_metadata['content-length'],
        )

        # if not called from command-line, assume called by update command
        if not self._called_from_command_line:
            # get the version that the update command is working on
            last_update_started = RawDataVersion.objects.filter(
                update_start_datetime__isnull=False
            ).latest('update_start_datetime')
            # confirm that update and download commands are working with same version
            if self.version != last_update_started:
                raise CommandError(
                    'Version available to download ({0}) does not match version '
                    'of update ({1}).'.format(
                        self.version,
                        last_update_started.release_datetime,
                    )
                )

        # log if a new version was found
        if created:
            logger.info('New CAL-ACCESS version available.')

        # if download is stalled, zip file is there, and user did not invoke restart
        if (
            self.version.download_stalled and
            os.path.exists(self.zip_path) and
            not options['restart']
        ):
            # enable resuming
            self.resume = True
            # set current size to partially downloaded zip
            self.local_file_size = os.path.getsize(self.zip_path)
            # set the datetime of last download to last modified date of zip file
            timestamp = os.path.getmtime(self.zip_path)
            self.local_file_datetime = datetime.fromtimestamp(timestamp, utc)
        else:
            self.resume = False
            self.local_file_size = 0
            self.local_file_datetime = None

        if not options['noinput']:
            # get downloaded versions
            downloaded_versions = RawDataVersion.objects.filter(
                download_finish_datetime__isnull=False
            )
            # if there are any
            if downloaded_versions:
                # get the last version downloaded datetime
                last_download_datetime = downloaded_versions.latest(
                    'download_finish_datetime'
                ).download_finish_datetime
                since_last_download = naturaltime(last_download_datetime)
            else:
                last_download_datetime = None

            # setting up the prompt
            prompt_context = dict(
                latest_version=self.version,
                resume=self.resume,
                local_file_size=size(self.local_file_size),
                local_file_datetime=self.local_file_datetime,
                download_dir=self.data_dir,
                since_previous_download=since_last_download,
            )

            prompt = render_to_string(
                'calaccess_raw/downloadcalaccessrawdata.txt',
                prompt_context,
            )

            # if the user doesn't confirm initially
            if not self.confirm_proceed(prompt):
                # but the user can resume
                if self.resume:
                    # prompt to restart
                    confirm_restart = self.confirm_proceed(
                        'Do you want re-start your update?\n'
                    )
                    # if user confirms restart, do not resume
                    if confirm_restart:
                        self.resume = False
                    else:
                        # if the user doesn't confirm restart, cancel
                        raise CommandError("Download cancelled")
                # if the user doesn't confirm initial and can't resume
                elif not self.resume:
                    # cancel
                    raise CommandError("Download cancelled")

        # if not resuming, store the download start time
        if not self.resume:
            self.version.download_start_datetime = now()
        # either way, reset the finish time
        self.version.download_finish_datetime = None
        # save here in case the command doesn't finish
        self.version.save()

        # check if local zip file is already completely downloaded before trying
        if self.local_file_size < self.version.expected_size:
            self.download()

        logger.debug('Download zip size: %s bytes' % os.path.getsize(self.zip_path))
        # log warning if downloaded zip size is not same as expected size
        if self.version.expected_size != os.path.getsize(self.zip_path):
            raise CommandError(
                'Expected {0} byte zip, but download {1} byte zip.'.format(
                    self.version.expected_size,
                    os.path.getsize(self.zip_path)
                )
            )

        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            self.archive()

        # store download finish time
        self.version.download_finish_datetime = now()
        # and save the RawDataVersion
        self.version.save()

    def download(self):
        """
        Download the ZIP file in pieces.
        """
        if self.verbosity:
            if self.resume:
                self.header(
                    "Resuming download of {:%m-%d-%Y %H:%M:%S} ZIP".format(
                        self.version.release_datetime
                    )
                )
            else:
                self.header(
                    "Downloading {:%m-%d-%Y %H:%M:%S} ZIP".format(
                        self.version.release_datetime
                    )
                )

        # Prep
        expected_size = self.version.expected_size
        headers = dict()
        if os.path.exists(self.zip_path):
            if self.resume:
                headers['Range'] = 'bytes=%d-' % self.local_file_size
                expected_size = expected_size - self.local_file_size
            else:
                os.remove(self.zip_path)

        # Stream the download
        chunk_size = 1024
        req = requests.get(self.url, stream=True, headers=headers)

        logger.debug('Server: %s <--  (from GET)' % req.headers['server'])
        logger.debug('ETag: %s <-- (from GET)' % req.headers['etag'])
        logger.debug('Last-Modified: %s <--  (from GET)' % req.headers['last-modified'])
        logger.debug('Content-Length: %s <-- (from GET)' % req.headers['content-length'])

        if self.download_metadata['etag'] != req.headers['etag']:
            raise CommandError("ETags in HEAD and GET requests don't match.")

        # in Python 2, need to convert this to long int
        try:
            divisor = long(chunk_size + 1)
        except NameError:
            # no long() in Python 3, all ints are long ints
            divisor = chunk_size + 1

        n_iters = float(expected_size) / divisor

        with open(self.zip_path, 'ab') as fp:
            for chunk in progress.bar(req.iter_content(chunk_size=chunk_size),
                                      expected_size=n_iters):
                fp.write(chunk)
                fp.flush()

    def archive(self):
        """
        Save a copy of the download zip file and each file inside.
        """
        if self.verbosity:
            self.log(" Archiving {0}".format(os.path.basename(self.zip_path)))
        # Remove previous zip file
        self.version.download_zip_archive.delete()
        # Store the actual download zip file size
        self.version.download_zip_size = os.path.getsize(self.zip_path)
        # Open up the zipped file so we can wrap it in the Django File obj
        zipped_file = open(self.zip_path, 'rb')
        # Save the zip on the raw data version
        self.version.download_zip_archive.save(
            self.url.split('/')[-1],
            File(zipped_file)
        )
        zipped_file.close()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-07-05 19:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0003_auto_20160705_1923'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rawdatafile',
            name='clean_file_size',
            field=models.IntegerField(default=0, help_text='Size of the .CSV file', verbose_name='size of clean data file in bytes'),
        ),
        migrations.AlterField(
            model_name='rawdatafile',
            name='download_file_size',
            field=models.IntegerField(default=0, help_text='Size of the .TSV file', verbose_name='size of raw data file in bytes'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for common models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_raw import models
from .base import BaseAdmin


@admin.register(models.FilernameCd)
class FilernameCdAdmin(BaseAdmin):
    """
    Custom admin for the FilernameCd model.
    """
    list_display = (
        "id",
        "xref_filer_id",
        "filer_id",
        "naml",
        "namf",
        "filer_type",
        "status",
        "effect_dt",
    )
    date_hierarchy = "effect_dt"


@admin.register(models.FilerFilingsCd)
class FilerFilingsCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerFilingsCd model.
    """
    list_display = (
        "id", "filer_id", "form_id",
        "filing_id", "filing_sequence"
    )


@admin.register(models.FilingsCd)
class FilingsCdAdmin(BaseAdmin):
    """
    Custom admin for the FilingsCd model.
    """
    list_display = ("filing_id", "filing_type")


@admin.register(models.SmryCd)
class SmryCdAdmin(BaseAdmin):
    """
    Custom admin for the SmryCd model.
    """
    list_display = (
        'filing_id',
        'amend_id',
        'form_type',
        'line_item',
        'amount_a',
        'amount_b',
        'amount_c',
    )


@admin.register(models.SpltCd)
class SpltCdAdmin(BaseAdmin):
    """
    Custom admin for the SpltCd model.
    """
    list_display = (
        "filing_id",
        "amend_id",
        "line_item",
        "pform_type",
        "elec_date",
        "elec_amount",
    )
    date_hierarchy = "elec_date"


@admin.register(models.TextMemoCd)
class TextMemoCdAdmin(BaseAdmin):
    """
    Custom admin for the TextMemoCd model.
    """
    list_display = ("filing_id", "amend_id", "form_type")


@admin.register(models.CvrE530Cd)
class CvrE530CdAdmin(BaseAdmin):
    """
    Custom admin for the CvrE530Cd model.
    """
    list_display = ("filing_id", "filer_naml", "office_cd", "cand_naml", "pmnt_dt", "pmnt_amount")
    date_hierarchy = "pmnt_dt"


@admin.register(models.AcronymsCd)
class AcronymsCdAdmin(BaseAdmin):
    """
    Custom admin for the AcronymsCd model.
    """
    list_display = ("acronym", "stands_for", "effect_dt", "a_desc")
    date_hierarchy = 'effect_dt'


@admin.register(models.AddressCd)
class AddressCdAdmin(BaseAdmin):
    """
    Custom admin for the AddressCd model.
    """
    list_display = ("adrid", "city", "st", "zip4",)


@admin.register(models.FilersCd)
class FilersCdAdmin(BaseAdmin):
    """
    Custom admin for the FilersCd model.
    """
    list_display = ("filer_id",)


@admin.register(models.FilerAcronymsCd)
class FilerAcronymsCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerAcronymsCd model.
    """
    list_display = ("acronym", "filer_id")


@admin.register(models.FilerAddressCd)
class FilerAddressCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerAddressCd model.
    """
    list_display = (
        "filer_id",
        "adrid",
        "effect_dt",
        "add_type"
    )


@admin.register(models.FilerEthicsClassCd)
class FilerEthicsClassCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerEthicsClassCd model.
    """
    list_display = ("filer_id", "session_id", "ethics_date",)
    date_hierarchy = "ethics_date"


@admin.register(models.FilerInterestsCd)
class FilerInterestsCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerInterestsCd model.
    """
    list_display = (
        "filer_id",
        "session_id",
        "interest_cd",
        "effect_date"
    )
    date_hierarchy = "effect_date"


@admin.register(models.FilerLinksCd)
class FilerLinksCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerLinksCd model.
    """
    list_display = (
        "filer_id_a",
        "filer_id_b",
        "link_type",
        "active_flg",
        "effect_dt",
        "termination_dt",
    )


@admin.register(models.FilerStatusTypesCd)
class FilerStatusTypesCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerStatusTypesCd model.
    """
    list_display = ("status_type", "status_desc")


@admin.register(models.FilerToFilerTypeCd)
class FilerToFilerTypeCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerToFilerTypeCd model.
    """
    list_display = (
        "filer_id",
        "filer_type",
        "effect_dt",
        "active",
        "session_id",
        "race",
        "district_cd",
        "party_cd"
    )
    date_hierarchy = "effect_dt"


@admin.register(models.FilerTypesCd)
class FilerTypesCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerTypesCd model.
    """
    list_display = (
        "filer_type",
        "description",
        "grp_type",
        "calc_use",
        "grace_period",
    )


@admin.register(models.FilerXrefCd)
class FilerXrefCdAdmin(BaseAdmin):
    """
    Custom admin for the FilerXrefCd model.
    """
    list_display = ("filer_id", "xref_id", "effect_dt", "migration_source")
    date_hierarchy = "effect_dt"


@admin.register(models.FilingPeriodCd)
class FilingPeriodCdAdmin(BaseAdmin):
    """
    Custom admin for the FilingPeriodCd model.
    """
    list_display = ("period_id", "start_date", "end_date", "period_desc")


@admin.register(models.GroupTypesCd)
class GroupTypesCdAdmin(BaseAdmin):
    """
    Custom admin for the GroupTypesCd model.
    """
    list_display = ("grp_id", "grp_name", "grp_desc")


@admin.register(models.HeaderCd)
class HeaderCdAdmin(BaseAdmin):
    """
    Custom admin for the HeaderCd model.
    """
    list_display = ("line_number", "form_id", "rec_type",)


@admin.register(models.HdrCd)
class HdrCdAdmin(BaseAdmin):
    """
    Custom admin for the HdrCd model.
    """
    list_display = ("filing_id", "amend_id", "cal_ver", "soft_name", "soft_ver",)


@admin.register(models.ImageLinksCd)
class ImageLinksCdAdmin(BaseAdmin):
    """
    Custom admin for the ImageLinksCd model.
    """
    list_display = ("img_link_id", "img_link_type", "img_id", "img_type", "img_dt")
    date_hierarchy = "img_dt"


@admin.register(models.LegislativeSessionsCd)
class LegislativeSessionsCdAdmin(BaseAdmin):
    """
    Custom admin for the LegislativeSessionsCd model.
    """
    list_display = ("session_id", "begin_date", "end_date")
    date_hierarchy = "begin_date"


@admin.register(models.LookupCodesCd)
class LookupCodesCdAdmin(BaseAdmin):
    """
    Custom admin for the LookupCodesCd model.
    """
    list_display = (
        "code_type",
        "code_id",
        "code_desc",
    )


@admin.register(models.NamesCd)
class NamesCdAdmin(BaseAdmin):
    """
    Custom admin for the NamesCd model.
    """
    list_display = ("namid", "namf", "naml",)


@admin.register(models.ReceivedFilingsCd)
class ReceivedFilingsCdAdmin(BaseAdmin):
    """
    Custom admin for the ReceivedFilingsCd model.
    """
    list_display = ("filing_id", "filer_id", "form_id", "received_date",)
    date_hierarchy = "received_date"


@admin.register(models.ReportsCd)
class ReportsCdAdmin(BaseAdmin):
    """
    Custom admin for the ReportsCd model.
    """
    list_display = ("rpt_name", "rpt_id", "rpt_type")



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-28 19:45
from __future__ import unicode_literals

from django.db import migrations


def migrate_command_datetimes(apps, schema_editor):
    """
    Migrate start_datetime and finish_datetime values from RawDataCommand
    """
    RawDataVersion = apps.get_model("calaccess_raw", "RawDataVersion")
    
    # for each version
    for v in RawDataVersion.objects.all():
        # get all downloads for the version
        downloads = v.command_logs.filter(
            command='downloadcalaccessrawdata',
        )
        # if there are downloads, get the last one started
        if downloads:
            d = downloads.latest('start_datetime')
            # set the version's download start and finish datetimes
            v.download_start_datetime = d.start_datetime
            v.download_finish_datetime = d.finish_datetime
        
        # get all extractions for the version
        extracts = v.command_logs.filter(
            command='extractcalaccessrawfiles',
        )
        # if there are extraction, get the last one started
        if extracts:
            e = extracts.latest('start_datetime')
            # set the version's extract start and finish datetimes
            v.extract_start_datetime = e.start_datetime
            v.extract_finish_datetime = e.finish_datetime

        # get all updates for the version
        updates = v.command_logs.filter(
            command__contains='update',
        )
        # if there are updates, get the last one started
        if updates:
            u = updates.latest('start_datetime')
            # set the version's update start and finish datetimes
            v.update_start_datetime = u.start_datetime
            v.update_finish_datetime = u.finish_datetime

        # save changes to the RawDataVersion
        v.save()

        # for each of the version's files
        for f in v.files.all():
            # get all the files clean jobs
            cleans = v.command_logs.filter(
                command='cleancalaccessrawfile',
                file_name=f.file_name,
            )
            # if there are clean jobs, get the last one started
            if cleans:
                c = cleans.latest('start_datetime')
                # set the file version's clean start and finish times
                f.clean_start_datetime = c.start_datetime
                f.clean_finish_datetime = c.finish_datetime

            # get all the files load jobs
            loads = v.command_logs.filter(
                command='loadcalaccessrawfile',
                file_name=f.file_name,
            )
            # if there are load jobs, get the last one started
            if loads:
                l = loads.latest('start_datetime')
                # set the file version's load start and finish times
                f.load_start_datetime = l.start_datetime
                f.load_finish_datetime = l.finish_datetime

            # save changes to the RawDataFile
            f.save()


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0011_auto_20160728_1944'),
    ]

    operations = [
        migrations.RunPython(migrate_command_datetimes)
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-26 14:25
from __future__ import unicode_literals

import calaccess_raw
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0009_rawdataversion_clean_zip_archive'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='rawdataversion',
            options={'get_latest_by': 'release_datetime', 'ordering': ('-release_datetime',), 'verbose_name': 'CAL-ACCESS raw data version'},
        ),
        migrations.RemoveField(
            model_name='rawdataversion',
            name='zip_file_archive',
        ),
        migrations.AddField(
            model_name='rawdataversion',
            name='download_zip_archive',
            field=models.FileField(blank=True, help_text='An archive of the original zipped file downloaded from CAL-ACCESS.', max_length=255, upload_to=calaccess_raw.archive_directory_path, verbose_name='download files zip file'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-06 17:19
from __future__ import unicode_literals

import calaccess_raw
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0005_auto_20160705_2145'),
    ]

    operations = [
        migrations.AddField(
            model_name='rawdatafile',
            name='error_log_archive',
            field=models.FileField(blank=True, help_text='An archive of the error log containing .TSV file lines that could not be cleaned and are excluded from the .CSV file.', max_length=255, upload_to=calaccess_raw.archive_directory_path, verbose_name='archive of error log'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for inactive models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_raw import models
from .base import BaseAdmin


@admin.register(models.BallotMeasuresCd)
class BallotMeasuresCdAdmin(BaseAdmin):
    """
    Custom admin for the BallotMeasuresCd model.
    """
    list_display = ("measure_name", "election_date", "jurisdiction")


@admin.register(models.CvrF470Cd)
class CvrF470CdAdmin(BaseAdmin):
    """
    Custom admin for the CvrF470Cd model.
    """
    list_display = ("filing_id", "filer_naml",)


@admin.register(models.FilerTypePeriodsCd)
class FilerTypePeriodsCd(BaseAdmin):
    """
    Custom admin for the FilerTypePeriodsCd model.
    """
    list_display = ("election_type", "filer_type", "period_id")


@admin.register(models.LobbyistContributions1Cd)
class LobbyistContributions1CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistContributions1Cd model.
    """
    list_display = (
        "filer_id",
        "filing_period_start_dt",
        "recipient_name",
        "amount"
    )
    date_hierarchy = "filing_period_start_dt"


@admin.register(models.LobbyistContributions2Cd)
class LobbyistContributions2CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistContributions2Cd model.
    """
    list_display = (
        "filer_id",
        "filing_period_start_dt",
        "recipient_name",
        "amount"
    )
    date_hierarchy = "filing_period_start_dt"


@admin.register(models.LobbyistContributions3Cd)
class LobbyistContributions3CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistContributions3Cd model.
    """
    list_display = (
        "filer_id",
        "filing_period_start_dt",
        "recipient_name",
        "amount"
    )
    date_hierarchy = "filing_period_start_dt"


@admin.register(models.LobbyistEmpLobbyist1Cd)
class LobbyistEmpLobbyist1CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistEmpLobbyist1Cd model.
    """
    list_display = (
        "lobbyist_id",
        "employer_id",
        "session_id",
        "lobbyist_first_name",
        "lobbyist_last_name",
        "employer_name",
    )


@admin.register(models.LobbyistEmpLobbyist2Cd)
class LobbyistEmpLobbyist2CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistEmpLobbyist2Cd model.
    """
    list_display = (
        "lobbyist_id",
        "employer_id",
        "session_id",
        "lobbyist_first_name",
        "lobbyist_last_name",
        "employer_name",
    )


@admin.register(models.LobbyistEmployer1Cd)
class LobbyistEmployer1CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistEmployer1Cd model.
    """
    list_display = (
        "employer_id",
        "session_id",
        "employer_name",
        "session_total_amt",
    )


@admin.register(models.LobbyistEmployer2Cd)
class LobbyistEmployer2CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistEmployer2Cd model.
    """
    list_display = (
        "employer_id",
        "session_id",
        "employer_name",
        "session_total_amt",
    )


@admin.register(models.LobbyistEmployer3Cd)
class LobbyistEmployer3CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistEmployer3Cd model.
    """
    list_display = (
        "employer_id",
        "session_id",
        "employer_name",
        "session_total_amt",
    )


@admin.register(models.LobbyistEmployerFirms1Cd)
class LobbyistEmployerFirms1CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistEmployerFirms1Cd model.
    """
    list_display = (
        "firm_id",
        "employer_id",
        "session_id",
        "firm_name",
        "termination_dt",
    )


@admin.register(models.LobbyistEmployerFirms2Cd)
class LobbyistEmployerFirms2CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistEmployerFirms2Cd model.
    """
    list_display = (
        "firm_id",
        "employer_id",
        "session_id",
        "firm_name",
        "termination_dt",
    )


@admin.register(models.LobbyistEmployerHistoryCd)
class LobbyistEmployerHistoryCdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistEmployerHistoryCd model.
    """
    list_display = ("employer_id", "employer_name")


@admin.register(models.LobbyistFirm1Cd)
class LobbyistFirm1CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistFirm1Cd model.
    """
    list_display = (
        "firm_id",
        "firm_name",
        "session_id",
        "contributor_id",
        "current_qtr_amt",
    )


@admin.register(models.LobbyistFirm2Cd)
class LobbyistFirm2CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistFirm2Cd model.
    """
    list_display = (
        "firm_id",
        "firm_name",
        "session_id",
        "contributor_id",
        "current_qtr_amt",
    )


@admin.register(models.LobbyistFirm3Cd)
class LobbyistFirm3CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistFirm3Cd model.
    """
    list_display = (
        "firm_id",
        "firm_name",
        "session_id",
        "contributor_id",
        "current_qtr_amt",
    )


@admin.register(models.LobbyistFirmEmployer1Cd)
class LobbyistFirmEmployer1CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistFirm2Cd model.
    """
    list_display = (
        "filing_id",
        "firm_name",
        "employer_name",
        "rpt_start",
        "per_total",
    )


@admin.register(models.LobbyistFirmEmployer2Cd)
class LobbyistFirmEmployer2CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistFirmEmployer2Cd model.
    """
    list_display = (
        "filing_id",
        "firm_name",
        "employer_name",
        "rpt_start",
        "per_total",
    )


@admin.register(models.LobbyistFirmHistoryCd)
class LobbyistFirmHistoryCdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistFirm2Cd model.
    """
    list_display = ("contributor_id", "firm_id", "session_id")


@admin.register(models.LobbyistFirmLobbyist1Cd)
class LobbyistFirmLobbyist1CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistFirm2Cd model.
    """
    list_display = (
        "lobbyist_id",
        "firm_id",
        "session_id",
        "lobbyist_first_name",
        "lobbyist_last_name",
        "firm_name"
    )


@admin.register(models.LobbyistFirmLobbyist2Cd)
class LobbyistFirmLobbyist2CdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyistFirm2Cd model.
    """
    list_display = (
        "lobbyist_id",
        "firm_id",
        "session_id",
        "lobbyist_first_name",
        "lobbyist_last_name",
        "firm_name"
    )


@admin.register(models.EfsFilingLogCd)
class EfsFilingLogCdAdmin(BaseAdmin):
    """
    Custom admin for the EfsFilingLogCd model.
    """
    list_display = (
        "id",
        "vendor",
        "filing_date",
        "filingstatus",
        "form_type",
    )
    date_hierarchy = "filing_date"
